<?php
/**
 * SprechstundenPlugin.class.php
 *
 * @author      Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author      Ivn Marcos <ivan.marcos@gmx.net>
 * @author      Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @package     LiteraturPlugin
 * @license     http://www.gnu.org/licenses/gpl.html GPL Licence 3
 * @version     2.0
 */

$RELATIVE_PATH_CALENDAR = $GLOBALS['RELATIVE_PATH_CALENDAR'];
$CALENDAR_DRIVER = $GLOBALS['CALENDAR_DRIVER'];

# Imports
require_once $GLOBALS['STUDIP_BASE_PATH'].'/'.$GLOBALS['RELATIVE_PATH_CALENDAR'].'/calendar_visual.inc.php';
require_once 'lib/classes/UserManagement.class.php';
require_once 'lib/classes/Table.class.php';

require_once 'classes/SprechstundenDB.class.php';
require_once 'classes/TermsDescription.class.php';
require_once 'classes/Term.class.php';
require_once 'classes/Slot.class.php';
require_once 'classes/StudentReservation.class.php';


/**
 * Hauptklasse des Plugins
 *
 */
class SprechstundenPlugin extends StudIPPlugin implements HomepagePlugin
{
    private $template_factory, $path, $username, $message, $links_per_page;
    public $user, $perm;
    private $ownpage = false;


    /**
     * Konstruktor
     *
     */
    public function __construct()
    {
        global  $user, $perm, $auth;
        parent::__construct();

        $this->user = $user;
        $this->perm = $perm;
        $this->path = $this->getPluginURL();

        //eintrge ro seite
        $this->links_per_page = (get_config('ENTRIES_PER_PAGE')) ? get_config('ENTRIES_PER_PAGE') : 25;
        $this->username = (Request::get('username'))? Request::get('username') : $auth->auth['uname'];
        $this->icon = 'icons/16/grey/consultation.png';
        $this->template_factory = new Flexi_TemplateFactory(dirname(__FILE__).'/templates');
        $this->layout =  $GLOBALS['template_factory']->open('layouts/base_without_infobox');
        $this->showNavigation();
    }

    /**
     * Navigationselemente erzeugen und so nur fr Lehrende
     *
     */
    public function showNavigation()
    {
        global $auth;

        // Dozent geht auf die eigene Homepage => Reiternavigation anzeigen
        if ($this->perm->have_perm('dozent') && !$this->perm->have_perm('admin') && (!Request::get('username') || Request::get('username') == $auth->auth['uname'])) {
            $navigation = new AutoNavigation(_("Sprechstunden"), PluginEngine::getURL($this, array(), 'show'));
            $navigation->addSubNavigation('show', new AutoNavigation('bersicht', PluginEngine::getURL($this, array(), 'show')));
            $navigation->addSubNavigation('newTerm', new AutoNavigation('Termine anlegen', PluginEngine::getURL($this, array(), 'newTerm')));
            Navigation::addItem('/profile/consultation', $navigation);
            Navigation::addItem('/start/homepage/consultation', $navigation);
            $this->ownpage = true;
        }
    }

    /**
     * Zeigt die Sprechstundentermine auf der Homepage in einer Box an
     *
     * @return string
     */
    public function getHomepageTemplate($user_id)
    {
        if ($this->getTeacherID()) {
            $template = $this->view('all', true);
            $template->title = _("Sprechstundentermine");
            $template->icon_url = 'icons/16/white/consultation.png';
            //eigene homepage
            if ($this->ownpage) {
                $template->admin_url = PluginEngine::getURL($this, array(), 'show');
                $template->admin_title = _('Zur Verwaltung der Sprechstundentermine');
            }
            return $template;
        }
        return NULL;

    }

    /**
     * Hauptfunktion zum Anzeigen des Plugins
     * Dient als Frontcontroller
     *
     */
    public function show_action()
    {
        # Dozent ist eingeloggt
        if ($this->perm->have_perm('dozent')) {
            Navigation::activateItem('/profile/consultation/show');
            if (!Request::get('term_id')) {
                $template = $this->template_factory->open('dozent_blocks');
                $template->set_layout($this->layout);
                $template->set_attribute('path', $this->path);
                $template->set_attribute('message', $this->message);
                unset($this->message);
                $template->set_attribute('blocks', SprechstundenDB::getSprechstundenBloecke($this->user->id));
                $template->set_attribute('link_newterm', PluginEngine::getURL($this,array(),'newTerm'));
                $template->set_attribute('link_deleteblocks', PluginEngine::getURL($this,array(),'deleteTerms'));
                $template->set_attribute('link', PluginEngine::getURL($this,array(),'show'));
                $template->set_attribute('link_slot', PluginEngine::getURL($this,array(),'slot'));
                $template->view_slots = $this->view(Request::get('term_id', 'all'));
                echo $template->render();
            } else {
                echo $this->view(Request::get('term_id', 'all'));
            }

        }
        // Student ist eingeloggt
        elseif ($this->perm->have_perm("autor") && !$this->perm->have_perm("dozent")) {
            echo $this->view(Request::get('term_id', 'all'));
        }
        // alle anderen flle
        else {
            throw new AccessDeniedException(_("Dieses Plugin steht Ihnen nicht zur Verfgung."));
        }
    }

    /**
     * Zeigt das Formular, um neue Sprechstundenblcke anzulegen
     *
     */
    public function newTerm_action()
    {
        $this->checkRights();

        // start dates
        $start = Request::get('start', date("d.m.Y", time()+(24*3600*7)));
        $ende = Request::get('ende', date("d.m.Y",  time()+(24*3600*7*4)+(3*3600)));
        $start_hour = Request::get('start_hour', date('H', time()));
        $start_minute = Request::get('start_minute', date('i', time()));
        $ende_hour = Request::get('ende_hour', date('H', time()+3600));
        $ende_minute = Request::get('ende_minute', date('i', time()));

    	if(Request::get('action') == 'deletereally'){
        	$db = DBManager::get()->prepare("DELETE FROM SprechstundenTerminDesc WHERE dozent_id = ? AND start_date = ? AND end_date= ?");
           	$db->execute(array($this->user->id, Request::int('start_stamp'), Request::int('end_stamp')));
           	$this->message = Messagebox::success('Der alte Termin wurde erfolgreich entfernt, bitte erstellen Sie jetzt den neuen Termin.');
       	} elseif (Request::submitted('speichern')) {
            $errors = $this->checkForm();
            if (count($errors) == 0) {
                $termDescription = new TermsDescription();
                //block anlegen
                $return = $termDescription->insert($this->user->id);
                if (isset($return['error'])) {
                    $errors = $return['error'];
                } elseif (isset($return['success'])) {
                    $this->message = Messagebox::success($return['success']);
                    $this->show_action();
                    return;
                }
            }
        }

        //Ausgabe
        $template = $this->template_factory->open('dozent_newterm');
        $template->set_layout($GLOBALS['template_factory']->open('layouts/base'));
        $template->set_attribute('start', $start);
        $template->set_attribute('start_hour', $start_hour);
        $template->set_attribute('start_minute', $start_minute);
        $template->set_attribute('ende', $ende);
        $template->set_attribute('ende_hour', $ende_hour);
        $template->set_attribute('ende_minute', $ende_minute);
        $template->set_attribute('errors', $errors);
        $template->set_attribute('link_back', PluginEngine::getURL($this, array(), 'show'));
        $template->set_attribute('link_save', PluginEngine::getURL($this, array(), 'newTerm'));
        $template->message = $this->message;
        $template->infobox = $this->getInfobox('1');
        echo $template->render();
    }

    /**
     * Enter description here...
     */
    public function deleteTerms_action()
    {
        $this->checkRights();

        Navigation::activateItem('/profile/consultation/show');

        //Wirklich lschen
        if(Request::get('action') == 'delete') {

            //min ein oder mehrere ids.
            if (Request::int('term_id')) {
                Request::set('term_ids', array(Request::int('term_id')));
                Request::set('term_id', NULL);
            } elseif (Request::intArray('term_ids')) {
                $this->message = Messagebox::error(_("Bitte whlen sie mindesten einen Sprechstundenterminblock zum Lschen aus."));
            }

            foreach (Request::intArray('term_ids') as $term_id) {
                $term = new Term($term_id);
                if (SprechstundenDB::checkTermin($term, $this->user->id)) {
                    SprechstundenDB::deleteSprechstundenTermin($this->user->id, $term, Request::get('reason'));
                }
            }

            $this->message = MessageBox::success(_('Die Sprechstundenterminblcke wurden erfolgreich gelscht.'));
            $this->show_action();
            return;
        }

        // the teacher choose to delete one block
        if (Request::int('term_id')) {

            $term = new Term(Request::int('term_id'));
            if (SprechstundenDB::checkTermin($term, $this->user->id)) {

                $desc = new TermsDescription($term->desc_id);
                $headline .= date("d.m.Y",$term->start_stamp)
                            . " von " . date("H:i",$term->start_stamp)
                            . " bis " . date("H:i",$term->end_stamp)
                            . " (" . $desc->place . ")";
                $terms_info = '<input type="hidden" name="term_id" value="'.Request::int('term_id').'">';
            }
        }
        // the teacher choose to delete several blocks
        elseif (Request::intArray('term_ids')) {
            $headline = 'Mehrere Sprechstundenblcke lschen:';
            $terms_info;
            // loop
            foreach(Request::intArray('term_ids') as $term_id) {
                $term = new Term($term_id);
                if (SprechstundenDB::checkTermin($term, $this->user->id))
                {
                    $desc = new TermsDescription($term->desc_id);
                    $terms_info .= '<input type="hidden" name="term_ids[]" value="'.$term->id.'">';
                    $terms_info .= "\nTermin am " . date("d.m.Y",$term->start_stamp)
                                            . " von " . date("H:i",$term->start_stamp)
                                            . " bis " . date("H:i",$term->end_stamp)
                                            . " (" . $desc->place . ")<br>";
                }
            }
        }
        // no slot_id, no term_id, no term_ids[]
        else {
            $this->message = Messagebox::error(_("Bitte whlen sie mindesten einen Sprechstundenterminblock zum Lschen aus."));
            $this->show_action();
            return;
        }

        //Ausgabe
        $template = $this->template_factory->open('dozent_deleteterm');
        $template->set_layout($GLOBALS['template_factory']->open('layouts/base'));
        $template->set_attribute('headline', $headline);
        $template->set_attribute('termsinfo', $terms_info);
        $template->set_attribute('link_back', PluginEngine::getURL($this,array(), 'show'));
        $template->set_attribute('link_delete', PluginEngine::getURL($this,array(),'deleteTerms'));
        $template->set_attribute('infobox', $this->getInfobox('2'));
        echo $template->render();
    }

    /**
     * Bietet Funktionen zum Belegen von Terminen
     */
    public function bookSlot_action()
    {
        if ($this->ownpage) {
            Navigation::activateItem('/profile/consultation/show');
        }

        //Termin wirklich buchen
        if (Request::submitted('speichern')) {
            $this->updateSlot();
            $this->show_action();
            return;
        }

        $slot = new Slot(Request::int('slot_id'));
        $term = new Term($slot->term_id);
        $desc = new TermsDescription($term->desc_id);

        // Dozent belegt einen Slot
        if ($this->perm->have_perm('dozent')) {
            $template = $this->template_factory->open('dozent_book');
            $template->infobox = $this->getInfobox('3');

        // Student belegt Slot
        } else {
            $template = $this->template_factory->open('student_book');
            $template->infobox = $this->getInfobox('5');
            $template->set_attribute('link_profile', URLHelper::getLink('about.php?username='.Request::get('username')));
        }

        //Ausgabe
        $template->set_layout($GLOBALS['template_factory']->open('layouts/base'));
        $template->set_attribute('slotid', Request::int('slot_id'));
        $template->set_attribute('ort', $desc->place);
        $template->set_attribute('termin', date("d.m.Y",$slot->slotStartStamp()));
        $template->set_attribute('zeit', date("H:i",$slot->slotStartStamp()).' - '.date("H:i", $slot->slotEndStamp()));
        $template->set_attribute('link', PluginEngine::getURL($this,array(), 'show'));
        $template->set_attribute('link_book', PluginEngine::getURL($this,array(), 'bookSlot'));
        $template->set_attribute('teacher_username', SprechstundenDB::getTeacherUsername($this->getTeacherID()));
        echo $template->render();

    }

    /**
     * lscht sprechstundenreservierungen
     */
    public function cancelSlot_action()
    {
        if($this->ownpage) {
            Navigation::activateItem('/profile/consultation/show');
        }

        if(Request::submitted('speichern')) {
            $this->updateSlot();
            $this->show_action();
            return;
        }

        $slot = new Slot(Request::get('slot_id'));
        $term = new Term($slot->term_id);
        $desc = new TermsDescription($term->desc_id);

        $template = $this->template_factory->open('common_cancel');
        $template->set_layout($GLOBALS['template_factory']->open('layouts/base'));
        $template->infobox = $this->getInfobox('4');
        $template->set_attribute('slotid', Request::get('slot_id'));
        $template->set_attribute('ort', htmlReady($desc->place));
        $template->set_attribute('termin', date("d.m.Y",$slot->slotStartStamp()));
        $template->set_attribute('zeit', date("H:i",$slot->slotStartStamp()). " bis " . date("H:i",$slot->slotEndStamp()));
        $template->set_attribute('link', PluginEngine::getURL($this, array(), 'show'));
        if (!$this->ownpage) {
            $template->set_attribute('link_profile', URLHelper::getLink('about.php?username='.Request::get('username')));
        }
        $template->set_attribute('link_cancel', PluginEngine::getURL($this, array(), 'cancelSlot'));
        $template->set_attribute('teacher_username', SprechstundenDB::getTeacherUsername($this->getTeacherID()));
        echo $template->render();

    }

    /**
     * lscht einen sprechstundentermin-slot
     */
    public function deleteSlot_action()
    {
        Navigation::activateItem('/profile/consultation/show');

        $slot = new Slot(Request::int('slot_id'));
        $term = new Term($slot->term_id);
        $desc = new TermsDescription($term->desc_id);

        // nur eigene sprechstunden knnen gelscht werden
        if ($desc->teacher_id == $this->user->id) {
            // nur nicht belegte termine lschen
            if ($slot->occupied != 1 && $slot->id != -1) {
                $slot->delete();
                $this->message = MessageBox::success('Der Sprechstundentermin wurde erfolgreich gelscht.');
            } else {
                $this->message = MessageBox::error('Der Sprechstundentermin konnte nicht gelscht werden, da entweder der Termin bereits belegt ist oder dieser nicht mehr vorhanden ist.');
            }
        }
        $this->show_action();
    }

    private function getTeacherID()
    {
        global $auth;

        // Student ist eingeloggt
        if ($this->perm->have_perm("autor") && !$this->perm->have_perm("dozent")) {
            // Student ist auf einer fremden homepage,
            if (Request::get('requesteduser') && Request::get('requesteduser') != $auth->auth['uname']) {
                $teacher = SprechstundenDB::getTeacherID(Request::get('requesteduser'));
            } elseif(Request::get('username') && Request::get('username') != $auth->auth['uname']) {
                $teacher = SprechstundenDB::getTeacherID(Request::get('username'));
            }
        }
        //Dozent ist eingeloggt und geht auf eigene Seite
        elseif($this->ownpage) {
            $teacher = $this->user->id;
        }

        return $teacher;
    }

    /**
     *
     * Anzeige der Sprechstunden Termine direkt im Plugin
     *
     */
    private function view($term_id='all', $box = false)
    {
        $teacher = $this->getTeacherID();

        // alle Termine abfragen
        if ($term_id == 'all') {
            $results = SprechstundenDB::getAllTerms($teacher, Request::int('page', 0), $this->links_per_page);
            $countSlots = SprechstundenDB::countSlots($teacher);
            $viewall = true;
        }
        // nur ein bestimmten Termin abfragen
        else {
            $results = SprechstundenDB::getTerm($teacher, $term_id, Request::int('page', 0), $this->links_per_page);
            $viewall = false;
        }

        // Keine Termine vorhanden
        if(count($results) == 0) {
            return $this->noTermsView($box);
        }

        //viele Eintrge
        else {
            #print_r($results);
            $views = array();
            foreach ($results as $result)
            {
                $view= array();
                $term = new Term($result['termid']);
                $desc = new TermsDescription($term->desc_id);
                $slot = new Slot($result['slotid']);

                $view['start'] = $slot->slotStartStamp();
                $view['end'] = $slot->slotEndStamp();
                $view['place'] = $desc->place;

                // Terminslot belegt
                if ($slot->occupied == 1)
                {
                    $booking = new StudentReservation(SprechstundenDB::getSprechstundenAnmeldungID($slot->id, $term->id));
                    $student_class = new UserManagement($booking->student_id);

                    $view['color'] = "red";
                    $view['status'] = "belegt";
                    $view['student'] = '<a href="'.URLHelper::getLink("about.php", array("username" => $student_class->user_data["auth_user_md5.username"])) . '">'
                                        . htmlReady($student_class->user_data["auth_user_md5.Vorname"] . " "
                                        . $student_class->user_data["auth_user_md5.Nachname"])
                                        . "</a>";
                    $view['remark'] = $booking->remark;
                    $view['slotid'] = $result['slotid'];


                    // teacher, my own homepage --> show method
                    if ($this->perm->have_perm('dozent'))
                    {
                        $view['option'] = 'cancel';
                    }
                    // STUDENTS --> TUTOR --> I AM LOGGED IN AND IT IS MY BOOKING
                    elseif ($booking->student_id == $this->user->id)
                    {
                        $view['color'] = "blue";
                        $view['option'] = 'cancel';
                    }
                    //Student, Termin belegt, aber nicht der eigene
                    else
                    {
                        $view['student'] = "-";
                        $view['remark'] = "-";
                        $view['option'] = '';
                    }
                }
                //Sprechstunde ist frei
                else
                {
                    $view['color'] = "green";
                    $view['status'] = "frei";
                    $view['student'] = "-";
                    $view['remark'] = "-";
                    $view['slotid'] = $result['slotid'];
                    $view['option'] = 'create';
                }
                // teacher, my own homepage --> show method
                if ($this->perm->have_perm('dozent')) {
                    $view['delete'] = true;
                }
                array_push($views, $view);
            }
            //Ausgabe
            //Druckansicht a)
            if(Request::get('action') == 'print') {
                $template = $this->template_factory->open('print_view');
            }
            //Normalansicht
            else {
                $template = $this->template_factory->open('common_view');
                if($box == true) {
                    $template->set_attribute('box', true);
                }
                if((!$box && $term_id != 'all') || !$this->ownpage) {
                    $template->set_layout($this->layout);
                }
            }
            $template->set_attribute('views', $views);
            $template->set_attribute('viewall', $viewall);
            $template->set_attribute('page', Request::int('page', 0));
            $template->set_attribute('path', $this->path);
            $template->set_attribute('singletermid', $term_id);
            $template->set_attribute('teacher_name', SprechstundenDB::getTeacherName($teacher));
            $template->set_attribute('teacher_username', SprechstundenDB::getTeacherUsername($teacher));
            $template->set_attribute('count_slots', $countSlots);
            $template->set_attribute('message', $this->message);
            $template->set_attribute('entries_per_page', $this->links_per_page);
            $template->set_attribute('link_book', PluginEngine::getURL($this,array(), 'bookSlot'));
            $template->set_attribute('link_cancel', PluginEngine::getURL($this,array(), 'cancelSlot'));
            $template->set_attribute('link_delete', PluginEngine::getURL($this,array(), 'deleteSlot'));
            $template->set_attribute('link', PluginEngine::getURL($this,array(), 'show'));

            if($box) {
                return $template;
            } else {
                return $template->render();
            }
        }
    }

    /**
     * berprft die Eingaben auf Fehler
     *
     * @return $errors
     */
    private function checkForm()
    {
        $errors_html = array();
        $array_number_fields = array(
            'start'=> _('Startanmeldungstag'),
            'start_hour'=> _('Startanmeldungsstunden'),
            #'start_minute'=> _('Startanmeldungsminuten'),
            'ende'=>_('Endtanmeldungstag'),
            'ende_hour'=> _('Startanmeldungsstunden'),
            #'ende_minute'=> _('Startanmeldungsminuten'),
            'day_of_the_week'=> _('Am Wochentag'),
            'place'=>_('Ort'),
            'interval'=> _('Intervall'),
            'duration'=>_('Dauer eines Termins'),
        );

        // loop - no empty values and digits
        foreach( $array_number_fields as $number_field => $error_text ) {
            // check
            if (!Request::get($number_field)) {
                $errors_html[] = 'Bitte fllen Sie das Feld "' . $error_text . '" aus.';
            }
        }

        if (strtotime(Request::get('start')) > strtotime(Request::get('ende'))) {
            $errors_html[] = "Der Endtermin liegt vor dem Starttermin.";
        }

        if (Request::int('start_hour') > Request::int('ende_hour')) {
            $errors_html[] = "Der Endzeitpunkt liegt vor dem Startzeitpunkt.";
        }
        return($errors_html);
    }

    /**
     * berprft die Sprechstundenreservierung und speichert diese in die DB oder
     * fhrt eine Stornierung durch und informiert die Benutzer.
     *
     */
    private function updateSlot()
    {
        // teacher, my own homepage
        if ($this->perm->have_perm('dozent')) {
            $teacher_id = $this->user->id;
            $student_id = Request::get('user_id');

        // only student
        } elseif ($this->perm->have_perm("autor") && !$this->perm->have_perm("dozent")) {
            $back = array('<a href="'.URLHelper::getLink('about.php?username='.Request::get('username')).'">Zurck zum Profil</a>');
            $student_id = $this->user->id;
            $username = Request::get('username');
            $teacher_id = DBManager::get()->query("SELECT user_id FROM auth_user_md5 WHERE `username` = ".DBManager::get()->quote($username))->fetch(PDO::FETCH_COLUMN);
        }

        $slot = new Slot(Request::get('slot_id'));

        // it is ok and free and there is no student's reservation
        if(SprechstundenDB::checkSlot($slot, $this->user->id, 0) && !$slot->hasStudentIn()) {
            $studentReservation = new StudentReservation();
            // save - insert - calendar - update slot -> occupied
            if (!($studentReservation->save($slot->term_id, $slot->id, $this->user->id, $teacher_id, $student_id, Request::get('reason')))) {
                $this->message =  Messagebox::error("Fehler: studentReservation->save()");
                return;
            }
            //$this->message = Messagebox::success("Die Sprechstundenterminbelegung wurde erfolgreich gespeichert", ($back)?$back:'');
            $this->message = Messagebox::success("Die Sprechstundenterminbelegung wurde erfolgreich gespeichert");
        }
        // it is occupied !!! ---> it has to be free / and reservation cancelled
        elseif (SprechstundenDB::checkSlot($slot, $this->user->id, 1)) {

            $user_id = DBManager::get()->query("SELECT `user_id` FROM `SprechstundenAnmeldung` WHERE `zeitslot_id`= {$slot->id}")->fetch(PDO::FETCH_COLUMN);

            // if student is logged in --> check if it's was my reservation
            // and it is my own reservation
            if(($this->perm->have_perm("autor") && !$this->perm->have_perm("dozent")) && ($user_id != $student_id)) {
                $this->message =  MessageBox::error("Fehler: Sie haben nicht die erforderlichen Rechte");
                return;
            }

            // get the reservation
            $sql = "SELECT RESERVATION.id FROM `SprechstundenAnmeldung` as RESERVATION"
                 . " INNER JOIN `SprechstundenZeitSlot` as SLOT"
                 . " ON RESERVATION.zeitslot_id=SLOT.id AND SLOT.id= {$slot->id} AND SLOT.belegt='1'";
            $reservation_id = DBManager::get()->query($sql)->fetch(PDO::FETCH_COLUMN);
            $studentReservation = new StudentReservation($reservation_id);
            $studentReservation->cancel(Request::get('reason'), $this->user->id, $teacher_id, $studentReservation->student_id, $slot);
            // message
            $this->message =  MessageBox::success("Der Sprechstundentermin wurde erfolgreich storniert.");
        }
    }

    private function noTermsView($box = false)
    {
        $text = SprechstundenDB::getText($this->getTeacherID());
        if (!$text) {
            $text = _('Es sind keine Termine vorhanden. Bitte kontaktieren die den Dozenten direkt.');
        }

        //Dozentensicht
        if ($this->ownpage) {
            $template = $this->template_factory->open('dozent_noterms');

            //Dozent speichert individuellen Text
            if (Request::submitted('save_text')) {
                $template->message = Messagebox::success(_("Der Text wurde erfolgreich gespeichert."));
                $text = Request::get('sprechstundetext');
                SprechstundenDB::setText($this->getTeacherID(), $text);
            }
            //Dozent lscht individuellen Text
            if (Request::submitted('delete_text')) {
                $template->message = Messagebox::success(_("Der Text wurde erfolgreich gelscht."));
                SprechstundenDB::deleteText($this->getTeacherID());
            }
        }
        //Studentensicht
        else {
            $template = $this->template_factory->open('student_noterms');
            if (!$box) {
                $template->set_layout($this->layout);
                Navigation::activateItem('/profile/view');
            }

        }
        $template->set_attribute('text', $text);
        $template->set_attribute('link', PluginEngine::getURL($this, array(), 'show'));
        $template->set_attribute('path', $this->path);
        $template->set_attribute('box', $box);

        if ($box) {
            return $template;
        } else {
            return $template->render();
        }
    }

    private function checkRights() {
        //Rechte berprfen
        if(!$this->perm->have_perm('dozent')){
            throw new AccessDeniedException("Diese Funktion des Plugins steht Ihnen nicht zur Verfgung.");
        }
    }

    /**
     * Gibt je nach id die passende Infobox aus
     *
     * @param int $id
     * @return array $infobox
     */
    private function getInfobox($id = '1')
    {
        // Infobox
        if($id == '1')
        {
            $infobox = array(
                array("kategorie" => _("Hinweise:"),
                    "eintrag" => array(
                        array(
                            "icon" => 'icons/16/black/info.png',
                            "text" => 'Ein Sprechstundenblock besteht aus einer Menge aufeinander folgender Sprechstundentermine.'
                        ),
                        array(
                            "icon" => 'icons/16/black/info.png',
                            "text" => 'Sie knnen einen einzelnen Sprechstundenblock oder eine ganze Serie von Blcken anlegen.'
                        ),
                        array(
                            "icon" => 'icons/16/black/info.png',
                            "text" => 'Um einen einzelnen Sprechstundenblock anzulegen, whlen Sie den richtigen Wochentag und setzen Sie Anfangs- und Enddatum gleich. Sind Anfangs- und Enddatum unterschiedlich, wird ein Serienblock erzeugt.'
                        )
                    )
                )
            );
        }
        elseif($id == '2')
        {
            $infobox = array(
                array("kategorie" => _("Hinweise:"),
                    "eintrag" => array(
                        array(
                            "icon" => 'icons/16/black/info.png',
                            "text" => 'Der Grund fr die Absage des Termins wird als interne Nachricht verschickt.'
                        )
                    )
                )
            );
        }
        elseif($id == '3')
        {
            $infobox = array(
                array("kategorie" => _("Hinweise:"),
                    "eintrag" => array(
                        array(
                            "icon" => 'icons/16/black/info.png',
                            "text" => 'Geben Sie den Namen des Studierenden oder Tutors ein und klicken Sie auf <i>suchen</i>. Whlen Sie anschlieend die Person aus.'
                        ),
                        array(
                            "icon" => 'icons/16/black/info.png',
                            "text" => 'Tragen Sie einen Grund fr ihre Besprechung ein. Der Student wird per Email ber den Sprechstundentermin informiert.'
                        ),
                        array(
                            "icon" => 'icons/16/black/info.png',
                            "text" => 'Als Suchergebnisse werden nur Studierende und Tutoren eingezeigt.'
                        )
                    )
                )
            );
        }
            elseif($id == '4')
        {
            $infobox = array(
                array("kategorie" => _("Hinweise:"),
                    "eintrag" => array(
                        array(
                            "icon" => 'icons/16/black/info.png',
                            "text" => 'Der Grund fr die Absage des Termins wird als interne Nachricht verschickt.'
                        )
                    )
                )
            );
        }
        elseif($id == '5')
        {
            $infobox = array(
                array("kategorie" => _("Hinweise:"),
                    "eintrag" => array(
                        array(
                            "icon" => 'icons/16/black/info.png',
                            "text" => 'Bitte geben Sie einen Grund fr ihre Besprechung ein, damit sich der Dozent auf die Sprechstunde vorbereiten kann.'
                        )
                    )
                )
            );
        }
        return array('content' => $infobox, 'picture' => 'infobox/dates.jpg');
    }
}