<?php
/**
 * SprechstundenDB.class.php
 *
 * Diese Datei enthlt die Datenbankfunktionen des Plugins
 * Dieses Plugin basiert auf PHP5 und steht unter der GPL.
 *
 * @author  	Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @package 	IBIT_SprechstundenPlugin
 * @copyright 	IBIT 2008-2010
 * @version 	1.8.1
 */

//Imports
require_once 'Term.class.php';
require_once 'StudentReservation.class.php';
require_once 'Slot.class.php';

/**
 * Klasse mit statischen Funktionen zur Abfrage der DB
 *
 */
class SprechstundenDB
{
	/**
	 * Holt alle Sprechstundenblcke eines Dozenten aus der DB
	 *
	 * @param string $id
	 * @return array
	 */
	public static function getSprechstundenBloecke($id)
	{
		// alle blcke anzeigen seit gestern (86400 sekunden) anzeigen, d.h. ltere Termine werden ausgeblendet.
	    $yesterday = (time()-86400);
	    $bloecke = array();

		$results = DBManager::get()->query("SELECT term.id AS id,description.ort AS ort FROM `SprechstundenTermin` AS term"
                    . " INNER JOIN `SprechstundenTerminDesc` AS description"
                    . " ON description.id=term.desc_id"
                    . " WHERE (description.dozent_id='{$id}')"
                    . " AND (term.start_time>='{$yesterday}') ORDER BY term.start_time ASC")->fetchAll(PDO::FETCH_ASSOC);

        foreach ($results as $result){
            $term = new Term($result['id']);
            // nur termine auswhlen, die nicht lter als ein tag sind
            if ($term->start_stamp > $yesterday){
                $bloecke[] = array(
                    'id' => $result['id'],
                    'ort' => $result['ort'],
                    'date' => strftime("%A, %d.%m.%Y",$term->start_stamp),
                    'start_time' => date("H:i",$term->start_stamp),
                    'end_time' => date("H:i",$term->end_stamp)
                );
            }
        }

        return $bloecke;
	}

	/**
	 * Lscht einen Sprechstundentermin aus der DB und informiert die Benutzer
	 *
	 * @todo: PDO
	 *
	 * @param string $userid
	 * @param TermObject $term
	 * @param string $reason
	 */
	public static function deleteSprechstundenTermin($userid, $term, $reason)
	{
		$db = new DB_Seminar;
		// 1.
		$sql = "SELECT RESERVATION.id,RESERVATION.zeitslot_id FROM `SprechstundenAnmeldung` as RESERVATION"
													. " INNER JOIN `SprechstundenZeitSlot` as SLOT"
													. " ON RESERVATION.zeitslot_id=SLOT.id"
													. " INNER JOIN `SprechstundenTermin` as TERM"
													. " ON TERM.id=SLOT.termin_id"
													. " AND (TERM.id=%s)"
													. " AND (TERM.start_time>='%s')"
													. " AND (SLOT.belegt='1')";
		$db->queryf($sql, $term->id, (time()-86400));
		while ($db->next_record())
		{
			//Student und Dozent benachrichtigen, dass ausfllt
			$studentReservation = new StudentReservation($db->f('id'));
			$studentReservation->cancel($reason, $userid, $userid, $studentReservation->student_id, new Slot($db->f('zeitslot_id')));
		}

		// 2.- delete all the slots
		$deleteSlotsSQL = "DELETE FROM `SprechstundenZeitSlot` WHERE `termin_id`='%s';";
		$db->queryf($deleteSlotsSQL, $term->id);

		// 3.- delete Term
		$deleteTermSQL = "DELETE FROM `SprechstundenTermin` WHERE `id`='%s';";
		$db->queryf($deleteTermSQL, $term->id);

		// 4.- delete descriptions
		$sql = "SELECT count(*) AS count FROM `SprechstundenTermin` WHERE desc_id='%s';";
		$db->queryf($sql, $term->desc_id);
		if($db->next_record())
		{
			if ($db->f('count') == 0)
			{
				$db->queryf("DELETE FROM `SprechstundenTerminDesc` WHERE id='%s';", $term->desc_id);
			}
		}
	}

	/**
	 * berprft einen Termin
	 *
     * @todo: PDO
	 *
	 * @param TermObjekt $term
	 * @return boolean
	 */
	public static function checkTermin($term, $userid)
	{
	    global $perm;

		$check = false;
		$db = new DB_Seminar;
		/**
		 *
		 * check if a term :
		 *	.- belongs to a teacher
		 *  .- a teacher or admin is active
		 *	.- if the term is active
		 */
		// check
		if ($term->id > 0) {
			// --- GET THE ACTIVE SLOTS FOR THIS TEACHER
			$selectSQL = "SELECT DESCRIPTION.id FROM `SprechstundenTerminDesc` as DESCRIPTION"
									. " INNER JOIN `SprechstundenTermin` as TERM"
									. " ON DESCRIPTION.id=TERM.desc_id"
									. " WHERE TERM.start_time>='%s'"
									. " AND TERM.id='%s'";

			// I am teacher --> I have to check
			if ($perm->have_perm('dozent')) {
				$selectSQL .=	" AND DESCRIPTION.dozent_id='%s'";
				$db->queryf($selectSQL, (time()-86400), $term->id, $userid);
			} else {
				$db->queryf($selectSQL, (time()-86400), $term->id);
			}
			// slot_id doesn't belongs to the dozent
			if ($db->affected_rows() > 0) {
				$check = true;
			}
		}
		return $check;
	}

	/**
	 * Enter description here...
	 *
     * @todo: PDO
	 *
	 * @param unknown_type $slot
	 * @param unknown_type $permissions
	 * @param unknown_type $userid
	 * @param unknown_type $status
	 * @return boolean
	 */
	public static function checkSlot($slot, $userid, $status = -1)
	{
	    global $perm;

		$check = false;
		$db = new DB_Seminar;
		/**
		 *
		 * check if a slot :
		 *	.- belongs to a teacher
		 *	.- if it is active
		 *  .- has the status
		 */
		if ($slot->id > 0)
		{
			// --- GET THE ACTIVE SLOTS FOR THIS TEACHER
			$sql = "SELECT DESCRIPTION.id FROM `SprechstundenTerminDesc` as DESCRIPTION"
					. " INNER JOIN `SprechstundenTermin` as TERM"
					. " ON DESCRIPTION.id=TERM.desc_id"
					. " INNER JOIN `SprechstundenZeitSlot` as SLOT"
					. " ON TERM.id=SLOT.termin_id"
					. " WHERE (TERM.start_time>= '%s')"
					. " AND (SLOT.id='%s')";
			// I am teacher --> I have to check
			if ($perm->have_perm('dozent'))
			{
				if($status != -1)
				{
					$sql .=	" AND (DESCRIPTION.dozent_id='%s') AND (SLOT.belegt='%s')";
					$db->queryf($sql, (time()-86400), $slot->id, $userid, $status);
				}
				else
				{
					$sql .=	" AND (DESCRIPTION.dozent_id='%s')";
					$db->queryf($sql, (time()-86400), $slot->id, $userid);
				}
			}
			elseif($status != -1)
			{
				$sql .= " AND (SLOT.belegt='%s')";
				$db->queryf($sql, (time()-86400), $slot->id, $status);
			}
			// slot_id belongs to the dozent
			if ($db->next_record())
			{
				$check = true;
			}
		}
		return $check;
	}

    /**
     * Holt zu einem Benutzernamen die BenutzerID aus der DB und berprft dabei
     * ob der Benutzer auch Dozent ist.
     *
     * @param string $username
     * @return string ID
     */
    public static function getTeacherID($username)
    {
        return DBManager::get()->query("SELECT `user_id` FROM `auth_user_md5` WHERE `username`='{$username}' AND perms = 'dozent'")->fetchColumn();
    }


	/**
	 * Gibt alle Blcke mit Terminen zu einem Dozenten zurck
	 *
     * @todo: PDO
	 *
	 * @param string $userid
	 * @param int $page
	 * @param int $perPage
	 * @return array() $ids
	 */
	public static function getAllTerms($userid, $page, $perPage)
	{
		$db = new DB_Seminar;
		$results = array();
		$sql = "SELECT TERM.id AS termid, SLOT.id AS slotid FROM `SprechstundenTerminDesc` as DESCRIPTION"
				. " INNER JOIN `SprechstundenTermin` as TERM"
				. " ON DESCRIPTION.id=TERM.desc_id"
				. " INNER JOIN `SprechstundenZeitSlot` as SLOT"
				. " ON TERM.id=SLOT.termin_id"
				. " WHERE (DESCRIPTION.dozent_id='%s') AND (TERM.start_time>='%s')"
				. " ORDER BY TERM.start_time,SLOT.position ASC"
				. " LIMIT %s, %s;";
		$db->queryf($sql, $userid, (time()-86400), (intval($page) * $perPage), $perPage);
		while ($db->next_record())
		{
			$results[] = array('termid' => $db->f('termid'), 'slotid' => $db->f('slotid'));
		}
		return $results;
	}

	/**
	 * Gibt alle Termine zu einem Block eines Dozenten zurck
	 *
     * @todo: PDO
	 *
	 * @param string $userid
	 * @param string $termid
	 * @param int $page
	 * @param int $perPage
	 * @return array() $ids
	 */
	public static function getTerm($userid, $termid, $page, $perPage)
	{
		$db = new DB_Seminar;
		$results = array();
		$sql = "SELECT TERM.id AS termid, SLOT.id AS slotid FROM `SprechstundenTerminDesc` as DESCRIPTION"
				. " INNER JOIN `SprechstundenTermin` as TERM"
				. " ON DESCRIPTION.id=TERM.desc_id"
				. " INNER JOIN `SprechstundenZeitSlot` as SLOT"
				. " ON TERM.id=SLOT.termin_id"
				. " WHERE (DESCRIPTION.dozent_id='%s') AND (TERM.start_time>='%s') AND (TERM.id='%s')"
				. " ORDER BY TERM.start_time,SLOT.position ASC"
				. " LIMIT %s, %s;";
		$db->queryf($sql, $userid, (time()-86400), $termid, (intval($page) * $perPage), $perPage);
		while ($db->next_record())
		{
			$results[] = array('termid' => $db->f('termid'), 'slotid' => $db->f('slotid'));
		}
		return $results;
	}


	/**
	 * Enter description here...
	 *
     * @todo: PDO
	 *
	 * @param string $userid
	 * @return int
	 */
	public static function countSlots($userid)
	{
		$db = new DB_Seminar;
		$sql = "SELECT count(SLOT.id) AS count FROM `SprechstundenTerminDesc` as DESCRIPTION"
				. " INNER JOIN `SprechstundenTermin` as TERM"
				. " ON DESCRIPTION.id=TERM.desc_id"
				. " INNER JOIN `SprechstundenZeitSlot` as SLOT"
				. " ON TERM.id=SLOT.termin_id"
				. " WHERE (DESCRIPTION.dozent_id='%s') AND (TERM.start_time>='%s');";
		$db->queryf($sql, $userid, (time()-86400));
		if($db->next_record())
		{
			return $db->f("count");
		}
	}

	/**
	 * Gibt den Namen zu einem Dozenten zurck
	 *
	 * @param string $userid
	 * @return string
	 */
	public static function getTeacherName($userid)
	{
		return DBManager::get()->query("SELECT CONCAT(Vorname, ' ', Nachname) FROM auth_user_md5 WHERE `user_id` = '{$userid}' AND `perms` = 'dozent'")->fetch(PDO::FETCH_COLUMN);
	}

    /**
     * Gibt den Namen zu einem Dozenten zurck
     *
     * @param string $userid
     * @return string
     */
    public static function getTeacherUsername($userid)
    {
        return DBManager::get()->query("SELECT username FROM auth_user_md5 WHERE `user_id` = '{$userid}' AND `perms` = 'dozent';")->fetch(PDO::FETCH_COLUMN);
    }

	/**
	 * Sucht Studenten
	 *
	 * @rodo: PDO
	 *
	 * @param string $searchstring
	 * @return array $users
	 */
	public static function searchUser($searchstring)
	{
		$db = new DB_Seminar;
		$users = array();
		$sql = "SELECT `user_id`, `username`, `Vorname`, `Nachname` FROM `auth_user_md5`"
			. " WHERE `perms` IN ('tutor','autor')"
			. " AND ( `username` LIKE '%%%s%%'"
			. " OR `Vorname` LIKE '%%%s%%'"
			. " OR `Nachname` LIKE '%%%s%%') ORDER BY Nachname;";
		$db->queryf($sql, $searchstring, $searchstring, $searchstring);
		while($db->next_record())
		{
			$result = array(
				'user_id' => $db->f('user_id'),
				'username' => $db->f('username'),
				'vorname' => $db->f('Vorname'),
				'nachname' => $db->f('Nachname'),
			);
			$users[] = $result;
		}
		#print_r($users);
		return $users;
	}

    /**
     *
     * @param int $slotid
     * @param int $termid
     * @return int
     */
    public static function getSprechstundenAnmeldungID($slotid, $termid)
    {
        $sql = "SELECT id FROM `SprechstundenAnmeldung` WHERE (`zeitslot_id`='{$slotid}') AND (`termin_id`='{$termid}') LIMIT 1";
        return DBManager::get()->query($sql)->fetch(PDO::FETCH_COLUMN);
    }

    /**
     *
     * @param md5 $user_id
     * @return string
     */
    public static function getText($user_id)
    {
        $sql = "SELECT sprechzeiten FROM user_inst WHERE user_id = '{$user_id}' AND externdefault = 1";
        $text = DBManager::get()->query($sql)->fetch(PDO::FETCH_COLUMN);

        if (empty($text)) {
            $cfg = new UserConfig($user_id);
            $text = $cfg->getValue(null, 'sprechstundenplugin_text');
        }

        return $text;
    }

    /**
     *
     * @param md5 $user_id
     * @param string $text
     */
    public static function setText($user_id, $text)
    {
        $sql = "SELECT count(sprechzeiten) FROM user_inst WHERE user_id = '{$user_id}' AND externdefault = 1";
        $user_inst = DBManager::get()->query($sql)->fetch(PDO::FETCH_COLUMN);

        if ($user_inst > 0) {
            $sql = "UPDATE user_inst SET sprechzeiten = ? WHERE user_id = ? AND externdefault = 1";
            $db = DBManager::get()->prepare($sql);
            $db->execute(array($text, $user_id));
        } else {
            $cfg = new UserConfig($user_id);
            $cfg->setValue($text, null, 'sprechstundenplugin_text');
        }
    }

    /**
     *
     * @param md5 $user_id
     */
    public static function deleteText($user_id)
    {
        $sql = "SELECT count(sprechzeiten) FROM user_inst WHERE user_id = '{$user_id}' AND externdefault = 1";
        $user_inst = DBManager::get()->query($sql)->fetch(PDO::FETCH_COLUMN);

        if ($user_inst > 0) {
            $sql = "UPDATE user_inst SET sprechzeiten = '' WHERE user_id = ? AND externdefault = 1";
            $db = DBManager::get()->prepare($sql);
            $db->execute(array($user_id));
        }

        $cfg = new UserConfig($user_id);
        $cfg->unsetValue(null, 'sprechstundenplugin_text');
    }
}
